/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var Cookie = require('./cookie').Cookie
  , CookieJar = require('./cookiejar').CookieJar
  , cookieManager = require('./middleware/cookiemanager');

/**
 * Export implementation classes.
 */

exports.Cookie = Cookie;
exports.CookieJar = CookieJar;
exports.cookieManager = cookieManager;

/**
 * Library version.
 */

exports.version = '0.0.1';

/**
 * Parse a cookie header into a `CookieJar`.
 *
 * @param {String} header
 * @param {Object} (optional) options
 * @return {CookieJar}
 * @api public
 */

exports.parseCookie = function(header, options) {
  if (!header) return;

  var cookie
    , cookieJar = new CookieJar(options);

  header.split(/[;,] */).forEach(function(part) {
    var sep = part.indexOf('=')
      , key = part.substr(0, sep).trim().toLowerCase()
      , val = part.substr(sep + 1).trim()
      , attr;

    if (key[0] === '$') {
      if (cookie) attr = key.substr(1);
      else return;
    } else {
      if (cookie) cookieJar.set(cookie);
      cookie = new Cookie(key, false);
      attr = attr || 'value'
    }

    // Lazy parse value + memoize
    Object.defineProperty(cookie, attr, {
      get: function() {
        delete this[attr];

        if ('"' == val[0]) {
          val = val.slice(1, -1);
        } else {
          val = val.replace(/\+/g, ' ');
          try {
            val = decodeURIComponent(val);
          } catch (err) {
            if (!(err instanceof URIError)) {
              throw err;
            }
          }
          try {
            val = JSON.parse(val);
          } catch (err) {
            if (!(err instanceof SyntaxError)) {
              throw err;
            }
          }
        }

        return this[attr] = val;
      },
      enumerable: true,
      configurable: true
    });
  });

  if (cookie) cookieJar.set(cookie);

  return cookieJar;
};
