/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var cookies = require('../index')
  , CookieJar = require('../cookiejar').CookieJar;

/**
 * Parse _Cookie_ header, populate `req.cookies` with an object keyed with
 * the cookie names, and save the `CookieJar` to `res.cookies`.
 *
 * @return {Function}
 * @api public
 */

module.exports = function(options) {
  return function(req, res, next) {
    if (req.cookies) return next();

    req.cookies = {};
    res.cookies = cookies.parseCookie(req.headers.cookie, options);

    if (res.cookies) {
      res.cookies.saveCookieValues(req.cookies);
    } else {
      res.cookies = new CookieJar(options);
    }

    var writeHead = res.writeHead;
    res.writeHead = function(status, headers) {
      res.cookies.setHeader(req, res);
      res.writeHead = writeHead;
      return res.writeHead(status, headers);
    };

    next();
  }
};
