/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , cookies = require('../lib/index');

/**
 * Vows tests.
 */

vows.describe('Cookie').addBatch({
  'when parsing RFC 2109 Example 1.3': {
    topic: function() {
      return cookies.parseCookie('$Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"');
    },
    'should be ok': function(topic) {
      var cookie = topic.get('customer');
      assert.equal(cookie.value, 'WILE_E_COYOTE');
      assert.equal(cookie.path, '/acme');
    },
    'should match case-insensitively': function(topic) {
      var cookie = topic.get('CUSTOMER');
      assert.equal(cookie.value, 'WILE_E_COYOTE');
      assert.equal(cookie.path, '/acme');
    }
  },
  'when parsing RFC 2109 Example 1.5': {
    topic: function() {
      return cookies.parseCookie('$Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"; Part_Number="Rocket_Launcher_0001"; $Path="/acme"');
    },
    'should be ok': function(topic) {
      var customer = topic.get('customer')
        , partNumber = topic.get('part_number');
      assert.equal(customer.value, 'WILE_E_COYOTE');
      assert.equal(customer.path, '/acme');
      assert.equal(partNumber.value, 'Rocket_Launcher_0001');
      assert.equal(partNumber.path, '/acme');
    },
    'should match case-insensitively': function(topic) {
      var customer = topic.get('CUSTOMER')
        , partNumber = topic.get('PART_NUMBER');
      assert.equal(customer.value, 'WILE_E_COYOTE');
      assert.equal(customer.path, '/acme');
      assert.equal(partNumber.value, 'Rocket_Launcher_0001');
      assert.equal(partNumber.path, '/acme');
    }
  },
  'when parsing a bad cookie': {
    topic: function() {
      return cookies.parseCookie('Test=%abc');
    },
    'should be ok': function(topic) {
      var cookie = topic.get('test');
      assert.equal(cookie.value, '%abc');
    }
  },
  'when parsing no cookie': {
    'should be ok': function(topic) {
      assert.isUndefined(cookies.parseCookie(''));
    }
  }
}).export(module);
